/*******************************************************************************

MihPhoto - Photo viewer for multi-touch devices
Copyright (C) 2010 Mihai Paslariu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "TouchUI.h"
#include "Config.h"

TouchUI::TouchUI()
{
	_background.load(g_config.install_dir + "/images/back.png" );
	_xoffset = _yoffset = _old_xoffset = _old_yoffset = 0;
	_width = 0;
	_screen_width = _screen_height = 0;
}

TouchUI::~TouchUI()
{
	clean();
}

void TouchUI::draw( QPainter & painter )
{
	_screen_width = painter.window().width();
	_screen_height = painter.window().height();
	limitScroll();

	// draw background
	QRect rect(0,0,painter.window().width(),_background.height());
	painter.drawTiledPixmap( rect, _background );
	
	// draw icons
	for ( int i = 0; i < _items.count(); i++ )
	{
		TouchUIItem * t = _items[i];
		painter.drawImage( t->x1+_xoffset, t->y1+_yoffset, t->image );
	}
}

TouchUIAction TouchUI::getAction( int x, int y )
{
	x -= _xoffset;
	y -= _yoffset;
	for ( int i = 0; i < _items.count(); i++ )
	{
		TouchUIItem * t = _items[i];
		if ( x > t->x1 && x < t->x2 && y > t->y1 && y < t->y2 )
		{
			return t->action;
		}
	}
	return TOUCH_ACTION_NONE;
}

void TouchUI::addAction( TouchUIAction a, QString icon_name )
{
	int space = 20;
	if ( _items.count() > 0 )
		space += _items.back()->x2;

	TouchUIItem * ta = new TouchUIItem;
	ta->image.load( g_config.install_dir + "/icons/" + icon_name );
	if ( ta->image.isNull() )
	{
		delete ta;
		return;
	}
	ta->action = a;
	ta->x1 = space;
	ta->x2 = ta->x1 + ta->image.width();
	ta->y1 = 20;
	ta->y2 = ta->y1 + ta->image.height();

	if ( ta->x2 > _width ) _width = ta->x2;

	_items.push_back(ta);
}

void TouchUI::addSeparator( void )
{
	addAction( TOUCH_ACTION_NONE, "separator.png" );
}

void TouchUI::limitScroll( void )
{
	int min_x = - _width + _screen_width - 20;
	if ( min_x > 0 ) min_x = 0;
	if ( _xoffset < min_x ) _xoffset = min_x;

	int max_x = 0;
	if ( _xoffset > max_x ) _xoffset = max_x;
}

void TouchUI::clean( void )
{
	for ( int i = 0; i < _items.count(); i++ )
		delete _items[i];
	_items.clear();

	_xoffset = _yoffset = _old_xoffset = _old_yoffset = 0;
	_width = 0;
	_screen_width = _screen_height = 0;
}
